@@ -29,13 +29,12 @@
 #include "challenger-httpd_challenge.h"
 #include "challenger-httpd_config.h"
 #include "challenger-httpd_info.h"
-#include "challenger-httpd_main_css.h"
 #include "challenger-httpd_mhd.h"
 #include "challenger-httpd_setup.h"
 #include "challenger-httpd_solve.h"
 #include "challenger-httpd_token.h"
+#include "challenger-httpd_spa.h"
 #include "challenger_database_lib.h"
-#include <taler/taler_templating_lib.h>
 
 
 /**
@@ -116,6 +115,29 @@ char *CH_address_type;
 char *CH_auth_command;
 
 
+/**
+ * Function called first by MHD with the full URL.
+ *
+ * @param cls NULL
+ * @param full_url the full URL
+ * @param con MHD connection object
+ * @return our handler context
+ */
+static void *
+full_url_track_callback (void *cls,
+                         const char *full_url,
+                         struct MHD_Connection *con)
+{
+  struct CH_HandlerContext *hc;
+
+  hc = GNUNET_new (struct CH_HandlerContext);
+  GNUNET_async_scope_fresh (&hc->async_scope_id);
+  GNUNET_SCHEDULER_begin_async_scope (&hc->async_scope_id);
+  hc->full_url = GNUNET_strdup (full_url);
+  return hc;
+}
+
+
 /**
  * A client has requested the given url using the given method
  * (#MHD_HTTP_METHOD_GET, #MHD_HTTP_METHOD_PUT,
@@ -171,18 +193,18 @@ url_handler (void *cls,
     {
       .url = "/",
       .method = MHD_HTTP_METHOD_GET,
-      .handler = &CH_MHD_handler_root
+      .handler = &CH_spa_redirect
+    },
+    {
+      .url = "/webui/",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler = &CH_handler_spa,
     },
     {
       .url = "/agpl",
       .method = MHD_HTTP_METHOD_GET,
       .handler = &CH_handler_agpl
     },
-    {
-      .url = "/main.css",
-      .method = MHD_HTTP_METHOD_GET,
-      .handler = &CH_handler_main_css
-    },
     {
       .url = "/config",
       .method = MHD_HTTP_METHOD_GET,
@@ -235,16 +257,12 @@ url_handler (void *cls,
               "Handling %s request for `%s'\n",
               method,
               url);
-  if (NULL == hc)
+  if (NULL == hc->connection)
   {
     const char *correlation_id;
     bool found = false;
 
-    hc = GNUNET_new (struct CH_HandlerContext);
-    *con_cls = hc;
     hc->connection = connection;
-    GNUNET_async_scope_fresh (&hc->async_scope_id);
-    GNUNET_SCHEDULER_begin_async_scope (&hc->async_scope_id);
     /* We only read the correlation ID on the first callback for every client */
     correlation_id = MHD_lookup_connection_value (connection,
                                                   MHD_HEADER_KIND,
@@ -548,17 +566,19 @@ run (void *cls,
   (void) cfgfile;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Starting challenger-httpd\n");
-  if (GNUNET_OK !=
-      TALER_TEMPLATING_init ("."))
-  {
-    GNUNET_break (0);
-    return;
-  }
   go = TALER_MHD_GO_NONE;
   if (CH_challenger_connection_close)
     go |= TALER_MHD_GO_FORCE_CONNECTION_CLOSE;
   TALER_MHD_setup (go);
 
+  if (GNUNET_OK !=
+      CH_spa_init ())
+  {
+    global_ret = EXIT_NOTCONFIGURED;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (config,
                                            "CHALLENGER",
@@ -627,6 +647,11 @@ run (void *cls,
         return;
       }
     }
+    else
+    {
+      CH_restrictions = json_object ();
+      GNUNET_assert (NULL != CH_restrictions);
+    }
   }
 
   global_ret = EXIT_NOTCONFIGURED;
@@ -666,6 +691,8 @@ run (void *cls,
                           NULL, NULL,
                           &url_handler, NULL,
                           MHD_OPTION_LISTEN_SOCKET, fh,
+                          MHD_OPTION_URI_LOG_CALLBACK,
+                          &full_url_track_callback, NULL,
                           MHD_OPTION_NOTIFY_COMPLETED,
                           &handle_mhd_completion_callback, NULL,
                           MHD_OPTION_CONNECTION_TIMEOUT,
