@@ -74,6 +74,11 @@ struct CH_HandlerContext
    */
   const char *path;
 
+  /**
+   * Copy of our original full URL with query parameters.
+   */
+  char *full_url;
+
   /**
    * Request handler for this request.
    */
