@@ -24,6 +24,7 @@
 #include <taler/taler_templating_lib.h>
 #include "challenger-httpd_authorize.h"
 #include "challenger-httpd_common.h"
+#include "challenger-httpd_spa.h"
 
 
 /**
@@ -185,6 +186,9 @@ CH_handler_authorize (struct CH_HandlerContext *hc,
     uint32_t address_attempts_left;
     uint32_t pin_transmissions_left;
     uint32_t auth_attempts_left;
+    struct GNUNET_TIME_Absolute last_tx_time;
+
+    bool solved;
     enum GNUNET_DB_QueryStatus qs;
 
     /* authorize_start will return 0 if a 'redirect_uri' was
@@ -198,7 +202,9 @@ CH_handler_authorize (struct CH_HandlerContext *hc,
                                  &last_address,
                                  &address_attempts_left,
                                  &pin_transmissions_left,
-                                 &auth_attempts_left);
+                                 &auth_attempts_left,
+                                 &solved,
+                                 &last_tx_time);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
@@ -227,79 +233,67 @@ CH_handler_authorize (struct CH_HandlerContext *hc,
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       break;
     }
-    if ( (0 == address_attempts_left) &&
-         (0 == pin_transmissions_left) &&
-         (0 == auth_attempts_left) )
+    if (0 == CH_get_output_type (hc->connection))
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Refusing authorization: zero attempts left\n");
-      json_decref (last_address);
-      return TALER_MHD_redirect_with_oauth_status (
-        hc->connection,
-        redirect_uri,
-        state,
-        "unauthorized_client",
-        "client exceeded authorization attempts limit (too many addresses)",
-        NULL);
+      char *prev_full_url = hc->full_url;
+      const char *rparams = strchr (hc->full_url, '?');
+
+      if (NULL == rparams)
+        GNUNET_asprintf (&hc->full_url,
+                         "%s?nonce=%s",
+                         prev_full_url,
+                         hc->path);
+
+      else
+        GNUNET_asprintf (&hc->full_url,
+                         "%s&nonce=%s",
+                         prev_full_url,
+                         hc->path);
+
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Redirect before %s now `%s'\n",
+                  prev_full_url,
+                  hc->full_url);
+
+      GNUNET_free (prev_full_url);
+      return CH_spa_redirect (hc, NULL, 0);
     }
     {
-      enum GNUNET_GenericReturnValue ret;
       json_t *args;
-      char *form;
       struct MHD_Response *resp;
-      unsigned int http_status;
       MHD_RESULT res;
 
-      GNUNET_asprintf (&form,
-                       "enter-%s-form",
-                       CH_address_type);
       args = GNUNET_JSON_PACK (
-        GNUNET_JSON_pack_allow_null (
-          GNUNET_JSON_pack_object_incref ("restrictions",
-                                          CH_restrictions)),
         GNUNET_JSON_pack_bool ("fix_address",
                                0 == address_attempts_left),
-        GNUNET_JSON_pack_string ("nonce",
-                                 hc->path),
         GNUNET_JSON_pack_allow_null (
           GNUNET_JSON_pack_object_steal ("last_address",
                                          last_address)),
+        GNUNET_JSON_pack_bool ("solved",
+                               solved),
+        GNUNET_JSON_pack_uint64 ("pin_transmissions_left",
+                                 pin_transmissions_left),
+        GNUNET_JSON_pack_uint64 ("auth_attempts_left",
+                                 auth_attempts_left),
+        GNUNET_JSON_pack_timestamp ("retransmission_time",
+                                    GNUNET_TIME_absolute_to_timestamp (
+                                      GNUNET_TIME_absolute_add (
+                                        last_tx_time,
+                                        CH_validation_duration))),
         GNUNET_JSON_pack_uint64 ("changes_left",
                                  address_attempts_left)
         );
-      http_status = MHD_HTTP_OK;
-      if (0 == CH_get_output_type (hc->connection))
-      {
-        ret = TALER_TEMPLATING_build (
-          hc->connection,
-          &http_status,
-          form,
-          NULL,
-          NULL,
-          args,
-          &resp);
-        GNUNET_free (form);
-        json_decref (args);
-        if (GNUNET_SYSERR == ret)
-        {
-          GNUNET_break (0);
-          return MHD_NO;
-        }
-        GNUNET_break (GNUNET_OK == ret);
-      }
-      else
-      {
-        GNUNET_break (0 ==
-                      json_object_del (args,
-                                       "nonce"));
-        resp = TALER_MHD_make_json_steal (args);
-      }
+
+      GNUNET_break (0 ==
+                    json_object_del (args,
+                                     "nonce"));
+      resp = TALER_MHD_make_json_steal (args);
       GNUNET_break (MHD_YES ==
                     MHD_add_response_header (resp,
                                              MHD_HTTP_HEADER_CACHE_CONTROL,
                                              "no-store,no-cache"));
       res = MHD_queue_response (hc->connection,
-                                http_status,
+                                MHD_HTTP_OK,
                                 resp);
       MHD_destroy_response (resp);
       return res;
