@@ -25,7 +25,6 @@
 #include "challenger-httpd_challenge.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_templating_lib.h>
-#include <taler/taler_merchant_service.h>
 #include <taler/taler_signatures.h>
 #include "challenger-httpd_common.h"
 
@@ -152,6 +151,11 @@ struct ChallengeContext
    */
   bool retransmit;
 
+  /**
+   * Is the challenge already solved?
+   */
+  bool solved;
+
   /**
    * Did we do the DB interaction?
    */
@@ -338,7 +342,8 @@ send_tan (struct ChallengeContext *bc)
     MHD_RESULT mres;
 
     GNUNET_break (0);
-    GNUNET_DISK_pipe_close (p);
+    GNUNET_break (GNUNET_OK ==
+                  GNUNET_DISK_pipe_close (p));
     mres = reply_error (bc,
                         "internal-error",
                         MHD_HTTP_INTERNAL_SERVER_ERROR,
@@ -352,7 +357,8 @@ send_tan (struct ChallengeContext *bc)
   pipe_stdin = GNUNET_DISK_pipe_detach_end (p,
                                             GNUNET_DISK_PIPE_END_WRITE);
   GNUNET_assert (NULL != pipe_stdin);
-  GNUNET_DISK_pipe_close (p);
+  GNUNET_break (GNUNET_OK ==
+                GNUNET_DISK_pipe_close (p));
 
   GNUNET_asprintf (&msg,
                    "PIN: %u",
@@ -653,7 +659,8 @@ CH_handler_challenge (struct CH_HandlerContext *hc,
       &bc->pin_attempts_left,
       &bc->retransmit,
       &bc->client_redirect_uri,
-      &bc->address_refused);
+      &bc->address_refused,
+      &bc->solved);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
@@ -677,29 +684,46 @@ CH_handler_challenge (struct CH_HandlerContext *hc,
       break;
     }
     bc->db_finished = true;
+    if (bc->solved)
+    {
+      struct MHD_Response *response;
+      MHD_RESULT ret;
+
+      json_t *args = GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_string ("type",
+                                 "completed"),
+        GNUNET_JSON_pack_string ("redirect_url",
+                                 bc->client_redirect_uri)
+        );
+
+      response = TALER_MHD_make_json (args);
+
+      ret = MHD_queue_response (hc->connection,
+                                MHD_HTTP_OK,
+                                response);
+      MHD_destroy_response (response);
+      return ret;
+    }
     if (bc->address_refused)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Address changes exhausted address change limit for this process\n");
-      return TALER_MHD_redirect_with_oauth_status (
-        hc->connection,
-        bc->client_redirect_uri,
-        bc->state,
-        "unauthorized_client",
-        "client exceeded authorization attempts limit (too many addresses attempted)",
-        NULL);
+      return reply_error (bc,
+                          "unauthorized_client",
+                          MHD_HTTP_TOO_MANY_REQUESTS,
+                          TALER_EC_CHALLENGER_TOO_MANY_ATTEMPTS,
+                          "client exceeded authorization attempts limit (too many addresses attempted)");
+
     }
     if (0 == bc->pin_attempts_left)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Address changes exhausted PIN limit for this address\n");
-      return TALER_MHD_redirect_with_oauth_status (
-        hc->connection,
-        bc->client_redirect_uri,
-        bc->state,
-        "unauthorized_client",
-        "client exceeded authorization attempts limit (too many PINs)",
-        NULL);
+      return reply_error (bc,
+                          "unauthorized_client",
+                          MHD_HTTP_TOO_MANY_REQUESTS,
+                          TALER_EC_CHALLENGER_TOO_MANY_ATTEMPTS,
+                          "client exceeded authorization attempts limit (too many PINs)");
     }
 
     if (bc->retransmit)
@@ -730,15 +754,17 @@ CH_handler_challenge (struct CH_HandlerContext *hc,
                                bc->pin_attempts_left),
       GNUNET_JSON_pack_string ("nonce",
                                hc->path),
+      GNUNET_JSON_pack_string ("type",
+                               "created"),
       GNUNET_JSON_pack_object_incref ("address",
                                       bc->address),
       GNUNET_JSON_pack_bool ("transmitted",
                              bc->retransmit),
-      GNUNET_JSON_pack_string ("next_tx_time",
-                               GNUNET_TIME_absolute2s (
-                                 GNUNET_TIME_absolute_add (
-                                   bc->last_tx_time,
-                                   CH_validation_duration)))
+      GNUNET_JSON_pack_timestamp ("retransmission_time",
+                                  GNUNET_TIME_absolute_to_timestamp (
+                                    GNUNET_TIME_absolute_add (
+                                      bc->last_tx_time,
+                                      CH_validation_duration)))
       );
     http_status = MHD_HTTP_OK;
     if (0 == CH_get_output_type (hc->connection))
