@@ -68,8 +68,12 @@ CH_handler_config (struct CH_HandlerContext *hc,
                                "urn:net:taler:specs:challenger:c-reference"),
       GNUNET_JSON_pack_string ("name",
                                "challenger"),
+      GNUNET_JSON_pack_string ("address_type",
+                               CH_address_type),
+      GNUNET_JSON_pack_object_incref ("restrictions",
+                                      CH_restrictions),
       GNUNET_JSON_pack_string ("version",
-                               "1:1:1"));
+                               "2:0:0"));
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (response,
                                            MHD_HTTP_HEADER_EXPIRES,
