@@ -25,7 +25,6 @@
 #include "challenger-httpd_solve.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_templating_lib.h>
-#include <taler/taler_merchant_service.h>
 #include <taler/taler_signatures.h>
 
 
@@ -317,13 +316,11 @@ CH_handler_solve (struct CH_HandlerContext *hc,
       {
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                     "Client exhausted all chances to satisfy challenge\n");
-        return TALER_MHD_redirect_with_oauth_status (
-          hc->connection,
-          bc->client_redirect_uri,
-          bc->state,
-          "access_denied",
-          "users exhausted all possibilities of passing the check",
-          NULL);
+        return reply_error (bc,
+                            "access_denied",
+                            MHD_HTTP_TOO_MANY_REQUESTS,
+                            TALER_EC_CHALLENGER_TOO_MANY_ATTEMPTS,
+                            "users exhausted all possibilities of passing the check");
       }
 
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -332,6 +329,8 @@ CH_handler_solve (struct CH_HandlerContext *hc,
         TALER_JSON_pack_ec (TALER_EC_CHALLENGER_INVALID_PIN),
         GNUNET_JSON_pack_uint64 ("addresses_left",
                                  bc->addr_left),
+        GNUNET_JSON_pack_string ("type",
+                                 "pending"),
         GNUNET_JSON_pack_uint64 ("pin_transmissions_left",
                                  bc->pin_transmissions_left),
         GNUNET_JSON_pack_uint64 ("auth_attempts_left",
@@ -360,6 +359,7 @@ CH_handler_solve (struct CH_HandlerContext *hc,
   {
     struct MHD_Response *response;
     char *url;
+    unsigned int http_status;
 
     {
       char *client_secret;
@@ -406,13 +406,23 @@ CH_handler_solve (struct CH_HandlerContext *hc,
                                 client_scope,
                                 address,
                                 client_redirect_uri);
-        url_encoded = TALER_urlencode (client_state);
-        GNUNET_asprintf (&url,
-                         "%s?code=%s&state=%s",
-                         client_redirect_uri,
-                         code,
-                         url_encoded);
-        GNUNET_free (url_encoded);
+        if (NULL == client_state)
+        {
+          GNUNET_asprintf (&url,
+                           "%s?code=%s",
+                           client_redirect_uri,
+                           code);
+        }
+        else
+        {
+          url_encoded = TALER_urlencode (client_state);
+          GNUNET_asprintf (&url,
+                           "%s?code=%s&state=%s",
+                           client_redirect_uri,
+                           code,
+                           url_encoded);
+          GNUNET_free (url_encoded);
+        }
         GNUNET_free (code);
       }
       json_decref (address);
@@ -422,41 +432,60 @@ CH_handler_solve (struct CH_HandlerContext *hc,
       GNUNET_free (client_state);
     }
 
+    if (0 == CH_get_output_type (hc->connection))
     {
-      const char *ok = "Ok!";
+      {
+        const char *ok = "Ok!";
 
-      response = MHD_create_response_from_buffer (strlen (ok),
-                                                  (void *) ok,
-                                                  MHD_RESPMEM_PERSISTENT);
-    }
-    if (NULL == response)
-    {
-      GNUNET_break (0);
+        response = MHD_create_response_from_buffer (strlen (ok),
+                                                    (void *) ok,
+                                                    MHD_RESPMEM_PERSISTENT);
+      }
+      if (NULL == response)
+      {
+        GNUNET_break (0);
+        GNUNET_free (url);
+        return MHD_NO;
+      }
+      TALER_MHD_add_global_headers (response);
+      GNUNET_break (MHD_YES ==
+                    MHD_add_response_header (response,
+                                             MHD_HTTP_HEADER_CONTENT_TYPE,
+                                             "text/plain"));
+      if (MHD_NO ==
+          MHD_add_response_header (response,
+                                   MHD_HTTP_HEADER_LOCATION,
+                                   url))
+      {
+        GNUNET_break (0);
+        MHD_destroy_response (response);
+        GNUNET_free (url);
+        return MHD_NO;
+      }
+      http_status = MHD_HTTP_FOUND;
       GNUNET_free (url);
-      return MHD_NO;
     }
-    TALER_MHD_add_global_headers (response);
-    GNUNET_break (MHD_YES ==
-                  MHD_add_response_header (response,
-                                           MHD_HTTP_HEADER_CONTENT_TYPE,
-                                           "text/plain"));
-    if (MHD_NO ==
-        MHD_add_response_header (response,
-                                 MHD_HTTP_HEADER_LOCATION,
-                                 url))
+    else
     {
-      GNUNET_break (0);
-      MHD_destroy_response (response);
+      json_t *args;
+      args = GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_string ("type",
+                                 "completed"),
+        GNUNET_JSON_pack_string ("redirect_url",
+                                 url)
+        );
       GNUNET_free (url);
-      return MHD_NO;
+
+      response = TALER_MHD_make_json (args);
+
+      http_status = MHD_HTTP_OK;
     }
-    GNUNET_free (url);
 
     {
       MHD_RESULT ret;
 
       ret = MHD_queue_response (hc->connection,
-                                MHD_HTTP_FOUND,
+                                http_status,
                                 response);
       MHD_destroy_response (response);
       return ret;
