@@ -36,7 +36,9 @@ CH_PG_authorize_start (void *cls,
                        json_t **last_address,
                        uint32_t *address_attempts_left,
                        uint32_t *pin_transmissions_left,
-                       uint32_t *auth_attempts_left)
+                       uint32_t *auth_attempts_left,
+                       bool *solved,
+                       struct GNUNET_TIME_Absolute *last_tx_time)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
@@ -62,6 +64,10 @@ CH_PG_authorize_start (void *cls,
                                   pin_transmissions_left),
     GNUNET_PQ_result_spec_uint32 ("auth_attempts_left",
                                   auth_attempts_left),
+    GNUNET_PQ_result_spec_bool ("solved",
+                                solved),
+    GNUNET_PQ_result_spec_absolute_time ("last_tx_time",
+                                         last_tx_time),
     GNUNET_PQ_result_spec_end
   };
 
@@ -79,7 +85,9 @@ CH_PG_authorize_start (void *cls,
            "   address"
            "  ,address_attempts_left"
            "  ,pin_transmissions_left"
-           "  ,auth_attempts_left;");
+           "  ,GREATEST(0, auth_attempts_left) AS auth_attempts_left"
+           "  ,auth_attempts_left = -1 AS solved"
+           "  ,last_tx_time;");
   return GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                    "authorize_start_validation",
                                                    params,
