@@ -42,6 +42,8 @@
  * @param[out] address_attempts_left set to number of address changing attempts left for this address
  * @param[out] pin_transmissions_left set to number of times the PIN can still be re-requested
  * @param[out] auth_attempts_left set to number of authentication attempts remaining
+ * @param[out] solved set to true if the challenge is already solved
+ * @param[out] last_tx_time set to the last time when we (presumably) send a PIN to @a last_address; 0 if never sent
  * @return transaction status:
  *   #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT if the address was changed
  *   #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS if we do not permit further changes to the address (attempts exhausted)
@@ -57,7 +59,9 @@ CH_PG_authorize_start (void *cls,
                        json_t **last_address,
                        uint32_t *address_attempts_left,
                        uint32_t *pin_transmissions_left,
-                       uint32_t *auth_attempts_left);
+                       uint32_t *auth_attempts_left,
+                       bool *solved,
+                       struct GNUNET_TIME_Absolute *last_tx_time);
 
 
 #endif
