@@ -38,7 +38,8 @@ CH_PG_challenge_set_address_and_pin (
   uint32_t *auth_attempts_left,
   bool *pin_transmit,
   char **client_redirect_uri,
-  bool *address_refused)
+  bool *address_refused,
+  bool *solved)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_TIME_Absolute now
@@ -76,6 +77,8 @@ CH_PG_challenge_set_address_and_pin (
       NULL),
     GNUNET_PQ_result_spec_bool ("address_refused",
                                 address_refused),
+    GNUNET_PQ_result_spec_bool ("solved",
+                                solved),
     GNUNET_PQ_result_spec_end
   };
   enum GNUNET_DB_QueryStatus qs;
@@ -92,6 +95,7 @@ CH_PG_challenge_set_address_and_pin (
            ",out_auth_attempts_left AS auth_attempts_left"
            ",out_client_redirect_uri AS client_redirect_uri"
            ",out_address_refused AS address_refused"
+           ",out_solved AS solved"
            " FROM challenger_do_challenge_set_address_and_pin"
            " ($1,$2,$3,$4,$5);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
