@@ -27,6 +27,7 @@
 #include "challenger_database_lib.h"
 #include "pg_helper.h"
 #include "pg_client_add.h"
+#include "pg_client_modify.h"
 #include "pg_client_delete.h"
 #include "pg_info_get_token.h"
 #include "pg_token_add_token.h"
@@ -390,6 +391,8 @@ libchallenger_plugin_db_postgres_init (void *cls)
     = &postgres_rollback;
   plugin->client_add
     = &CH_PG_client_add;
+  plugin->client_modify
+    = &CH_PG_client_modify;
   plugin->client_delete
     = &CH_PG_client_delete;
   plugin->client_check
