@@ -81,7 +81,7 @@ struct CHALLENGER_DatabasePlugin
    * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
    */
   enum GNUNET_GenericReturnValue
-  (*drop_tables)(void *cls);
+    (*drop_tables)(void *cls);
 
 
   /**
@@ -91,7 +91,7 @@ struct CHALLENGER_DatabasePlugin
    * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
    */
   enum GNUNET_GenericReturnValue
-  (*create_tables)(void *cls);
+    (*create_tables)(void *cls);
 
 
   /**
@@ -105,7 +105,7 @@ struct CHALLENGER_DatabasePlugin
    *         #GNUNET_SYSERR on hard errors
    */
   enum GNUNET_GenericReturnValue
-  (*preflight)(void *cls);
+    (*preflight)(void *cls);
 
 
   /**
@@ -117,8 +117,8 @@ struct CHALLENGER_DatabasePlugin
    * @return #GNUNET_OK on success
    */
   enum GNUNET_GenericReturnValue
-  (*begin_transaction)(void *cls,
-                       const char *name);
+    (*begin_transaction)(void *cls,
+                         const char *name);
 
 
   /**
@@ -128,7 +128,7 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*commit_transaction)(void *cls);
+    (*commit_transaction)(void *cls);
 
 
   /**
@@ -148,8 +148,8 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*gc)(void *cls,
-        struct GNUNET_TIME_Absolute expire);
+    (*gc)(void *cls,
+          struct GNUNET_TIME_Absolute expire);
 
 
   /**
@@ -162,10 +162,26 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*client_add)(void *cls,
-                const char *client_url,
-                const char *client_secret,
-                uint64_t *client_id);
+    (*client_add)(void *cls,
+                  const char *client_url,
+                  const char *client_secret,
+                  uint64_t *client_id);
+
+
+  /**
+   * Modify client in the list of authorized clients.
+   *
+   * @param cls
+   * @param client_id the client ID on success
+   * @param client_url URL of the client
+   * @param client_secret authorization secret for the client, NULL to not modify the secret
+   * @return transaction status
+   */
+  enum GNUNET_DB_QueryStatus
+    (*client_modify)(void *cls,
+                     uint64_t client_id,
+                     const char *client_url,
+                     const char *client_secret);
 
   /**
    * Delete client from the list of authorized clients.
@@ -175,8 +191,8 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*client_delete)(void *cls,
-                   const char *client_url);
+    (*client_delete)(void *cls,
+                     const char *client_url);
 
 
   /**
@@ -192,11 +208,11 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*client_check)(void *cls,
-                  uint64_t client_id,
-                  const char *client_secret,
-                  uint32_t counter_increment,
-                  char **client_url);
+    (*client_check)(void *cls,
+                    uint64_t client_id,
+                    const char *client_secret,
+                    uint32_t counter_increment,
+                    char **client_url);
 
 
   /**
@@ -211,10 +227,10 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*setup_nonce)(void *cls,
-                 uint64_t client_id,
-                 const struct CHALLENGER_ValidationNonceP *nonce,
-                 struct GNUNET_TIME_Absolute expiration_time);
+    (*setup_nonce)(void *cls,
+                   uint64_t client_id,
+                   const struct CHALLENGER_ValidationNonceP *nonce,
+                   struct GNUNET_TIME_Absolute expiration_time);
 
 
   /**
@@ -233,22 +249,26 @@ struct CHALLENGER_DatabasePlugin
    * @param[out] address_attempts_left set to number change address operations left for this @a nonce
    * @param[out] pin_transmissions_left set to number of times the PIN can still be re-requested
    * @param[out] auth_attempts_left set to number of authentication attempts remaining
+   * @param[out] solved set to true if the challenge is already solved
+   * @param[out] last_tx_time set to the last time when we (presumably) send a PIN to @a last_address; 0 if never sent
    * @return transaction status:
    *   #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT if the address was changed
    *   #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS if we do not permit further changes to the address (attempts exhausted)
    *   #GNUNET_DB_STATUS_HARD_ERROR on failure
    */
   enum GNUNET_DB_QueryStatus
-  (*authorize_start)(void *cls,
-                     const struct CHALLENGER_ValidationNonceP *nonce,
-                     uint64_t client_id,
-                     const char *client_scope,
-                     const char *client_state,
-                     const char *client_redirect_uri,
-                     json_t **last_address,
-                     uint32_t *address_attempts_left,
-                     uint32_t *pin_transmissions_left,
-                     uint32_t *auth_attempts_left);
+    (*authorize_start)(void *cls,
+                       const struct CHALLENGER_ValidationNonceP *nonce,
+                       uint64_t client_id,
+                       const char *client_scope,
+                       const char *client_state,
+                       const char *client_redirect_uri,
+                       json_t **last_address,
+                       uint32_t *address_attempts_left,
+                       uint32_t *pin_transmissions_left,
+                       uint32_t *auth_attempts_left,
+                       bool *solved,
+                       struct GNUNET_TIME_Absolute *last_tx_time);
 
 
   /**
@@ -268,13 +288,14 @@ struct CHALLENGER_DatabasePlugin
    * @param[out] auth_attempts_left set to number of attempts the user has left on this pin
    * @param[out] client_redirect_uri redirection URI of the client (for reporting failures)
    * @param[out] address_refused set to true if the address was refused (address change attempts exhausted)
+   * @param[out] solved set to true if the challenge is already solved
    * @return transaction status:
    *   #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT if the address was changed
    *   #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS if we do not permit further changes to the address (attempts exhausted)
    *   #GNUNET_DB_STATUS_HARD_ERROR on failure
    */
   enum GNUNET_DB_QueryStatus
-  (*challenge_set_address_and_pin)(
+    (*challenge_set_address_and_pin)(
     void *cls,
     const struct CHALLENGER_ValidationNonceP *nonce,
     const json_t *address,
@@ -285,7 +306,8 @@ struct CHALLENGER_DatabasePlugin
     uint32_t *auth_attempts_left,
     bool *pin_transmit,
     char **client_redirect_uri,
-    bool *address_refused);
+    bool *address_refused,
+    bool *solved);
 
 
   /**
@@ -308,17 +330,17 @@ struct CHALLENGER_DatabasePlugin
    *   #GNUNET_DB_STATUS_HARD_ERROR on failure
    */
   enum GNUNET_DB_QueryStatus
-  (*validate_solve_pin)(void *cls,
-                        const struct CHALLENGER_ValidationNonceP *nonce,
-                        uint32_t new_pin,
-                        bool *solved,
-                        bool *exhausted,
-                        bool *no_challenge,
-                        char **state,
-                        uint32_t *addr_left,
-                        uint32_t *auth_attempts_left,
-                        uint32_t *pin_transmissions_left,
-                        char **client_redirect_uri);
+    (*validate_solve_pin)(void *cls,
+                          const struct CHALLENGER_ValidationNonceP *nonce,
+                          uint32_t new_pin,
+                          bool *solved,
+                          bool *exhausted,
+                          bool *no_challenge,
+                          char **state,
+                          uint32_t *addr_left,
+                          uint32_t *auth_attempts_left,
+                          uint32_t *pin_transmissions_left,
+                          char **client_redirect_uri);
 
 
   /**
@@ -339,13 +361,13 @@ struct CHALLENGER_DatabasePlugin
    *   #GNUNET_DB_STATUS_HARD_ERROR on failure
    */
   enum GNUNET_DB_QueryStatus
-  (*validation_get)(void *cls,
-                    const struct CHALLENGER_ValidationNonceP *nonce,
-                    char **client_secret,
-                    json_t **address,
-                    char **client_scope,
-                    char **client_state,
-                    char **client_redirect_uri);
+    (*validation_get)(void *cls,
+                      const struct CHALLENGER_ValidationNonceP *nonce,
+                      char **client_secret,
+                      json_t **address,
+                      char **client_scope,
+                      char **client_state,
+                      char **client_redirect_uri);
 
 
   /**
@@ -359,11 +381,11 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*token_add_token)(void *cls,
-                     const struct CHALLENGER_ValidationNonceP *nonce,
-                     const struct CHALLENGER_AccessTokenP *grant,
-                     struct GNUNET_TIME_Relative grant_expiration,
-                     struct GNUNET_TIME_Relative address_expiration);
+    (*token_add_token)(void *cls,
+                       const struct CHALLENGER_ValidationNonceP *nonce,
+                       const struct CHALLENGER_AccessTokenP *grant,
+                       struct GNUNET_TIME_Relative grant_expiration,
+                       struct GNUNET_TIME_Relative address_expiration);
 
 
   /**
@@ -377,11 +399,11 @@ struct CHALLENGER_DatabasePlugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*info_get_token)(void *cls,
-                    const struct CHALLENGER_AccessTokenP *grant,
-                    uint64_t *rowid,
-                    json_t **address,
-                    struct GNUNET_TIME_Timestamp *address_expiration);
+    (*info_get_token)(void *cls,
+                      const struct CHALLENGER_AccessTokenP *grant,
+                      uint64_t *rowid,
+                      json_t **address,
+                      struct GNUNET_TIME_Timestamp *address_expiration);
 
 
 };
